//-------+---------+---------+---------+---------+---------+---------+--------=
//
// File: gapped_extend.h
//
//----------

#ifndef gapped_extend_H			// (prevent multiple inclusion)
#define gapped_extend_H

// other files

#include "utilities.h"			// utility stuff
#include "segment.h"			// segment table management stuff
#include "edit_script.h"		// alignment edit script stuff

// establish ownership of global variables

#ifdef gapped_extend_owner
#define global
#else
#define global extern
#endif

// "deep link" control variable access

#ifdef gapped_extend_owner
int gapped_extend_verbosity = 0;	// ranges from 0 (no info) to 10 (everything)
int gapped_extend_inhibitTruncationReport = false;
int gapped_extend_dbgShowIdentity = false;
									// true => don't report alignment truncations
#else
global int gapped_extend_verbosity;
global int gapped_extend_inhibitTruncationReport;
global int gapped_extend_dbgShowIdentity;
#endif

//----------
//
// data structures and types
//
//----------

// traceback data structure

typedef struct tback
	{
	u32		size;				// the number of entries allocated for cell[]
	u8		space[1];			// the traceback cells
	} tback;

//----------
//
// statistics for events in this module
//
//----------

#if ((defined collect_stats) && (defined gapped_extend_owner))

struct
	{
	int64 numAnchors;
	int64 numAnchorsExtended;
	int64 numPeaks;
#if (scoreType == 'I')
	s64   totalPeakScore;
#else
	score totalPeakScore;
#endif // scoreType
	int64 numExtensions;
	int64 dpCellsVisited;
	int64 maxDpRows;
	int64 maxDpColumns;
	} gappedExtendStats;

// stats macros

#define gapped_extend_count_stat(field)   ++gappedExtendStats.field
#define gapped_extend_uncount_stat(field) --gappedExtendStats.field
#define gapped_extend_set_stat(field,val) (gappedExtendStats.field = val)
#define gapped_extend_add_stat(field,val) (gappedExtendStats.field += val)
#define gapped_extend_max_stat(field,val) if (val > gappedExtendStats.field) gappedExtendStats.field = val
#else
#define gapped_extend_count_stat(field)
#define gapped_extend_uncount_stat(field)
#define gapped_extend_set_stat(field,val)
#define gapped_extend_add_stat(field,val)
#define gapped_extend_max_stat(field,val)
#endif // collect_stats && gapped_extend_owner

// prototypes for stats routines

void gapped_extend_zero_stats    (void);
void gapped_extend_show_stats    (FILE* f);
void gapped_extend_generic_stats (FILE* f, void (*func) (FILE*, const char*, ...));

//----------
//
// prototypes for routines in gapped_extend.c
//
//----------

void     reduce_to_points    (seq* seq1, seq* seq2, scoreset* scoring,
                              segtable* anchors);
alignel* gapped_extend       (seq* seq1, u8* rev1, seq* seq2, u8* rev2,
                              int selfCompare,
                              scoreset* scoring, segtable* anchors, tback* tb,
                              score yDrop, sthresh scoreThresh);
tback*   new_traceback       (u32 size);
void     free_traceback      (tback* tb);
void     free_traceback_rows (void);

#undef global
#endif // gapped_extend_H
